"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const heroku_cli_util_1 = require("heroku-cli-util");
const graphql_1 = require("graphql");
const chalk_1 = __importDefault(require("chalk"));
const git_1 = require("../../git");
const Command_1 = require("../../Command");
const utils_1 = require("../../utils");
const graphqlTypes_1 = require("apollo-language-server/lib/graphqlTypes");
const moment_1 = __importDefault(require("moment"));
const formatChange = (change) => {
    let color = (x) => x;
    if (change.type === graphqlTypes_1.ChangeType.FAILURE) {
        color = chalk_1.default.red;
    }
    if (change.type === graphqlTypes_1.ChangeType.WARNING) {
        color = chalk_1.default.yellow;
    }
    return {
        type: color(change.type),
        code: color(change.code),
        description: color(change.description)
    };
};
function formatMarkdown({ checkSchemaResult, serviceName, tag }) {
    const { diffToPrevious } = checkSchemaResult;
    if (!diffToPrevious) {
        throw new Error("checkSchemaResult.diffToPrevious missing");
    }
    const { validationConfig } = diffToPrevious;
    if (!validationConfig) {
        throw new Error("checkSchemaResult.diffToPrevious.validationConfig missing");
    }
    const days = -moment_1.default()
        .add(validationConfig.from, "second")
        .diff(moment_1.default().add(validationConfig.to, "second"), "days");
    const breakingChanges = diffToPrevious.changes.filter(change => change.type === "FAILURE");
    return `
### Apollo Service Check
🔄 Validated your local schema against schema tag \'${tag}\' on service \'${serviceName}\'.
🔢 Compared **${diffToPrevious.changes.length} schema changes** against operations seen over the **last ${days === 1 ? "day" : `${days} days`}**.
${breakingChanges.length > 0
        ? `❌ Found **${diffToPrevious.changes.filter(change => change.type === "FAILURE")
            .length} breaking changes** that would affect **${diffToPrevious.affectedQueries
            ?
                diffToPrevious.affectedQueries.length
            : "no"} operations**`
        : `✅ Found **no breaking changes**.`}

🔗 [View your service check details](${checkSchemaResult.targetUrl}).
`;
}
exports.formatMarkdown = formatMarkdown;
class ServiceCheck extends Command_1.ProjectCommand {
    async run() {
        const { gitContext, checkSchemaResult, config, shouldOutputJson, shouldOutputMarkdown } = await this.runTasks(({ config, flags, project }) => [
            {
                title: "Checking service for changes",
                task: async (ctx) => {
                    if (!config.name) {
                        throw new Error("No service found to link to Engine");
                    }
                    const tag = flags.tag || config.tag || "current";
                    const schema = await project.resolveSchema({ tag });
                    ctx.gitContext = await git_1.gitInfo(this.log);
                    const historicParameters = utils_1.validateHistoricParams({
                        validationPeriod: flags.validationPeriod,
                        queryCountThreshold: flags.queryCountThreshold,
                        queryCountThresholdPercentage: flags.queryCountThresholdPercentage
                    });
                    ctx.checkSchemaResult = await project.engine.checkSchema(Object.assign({ id: config.name, schema: graphql_1.introspectionFromSchema(schema).__schema, tag: flags.tag, gitContext: ctx.gitContext, frontend: flags.frontend || config.engine.frontend }, (historicParameters && { historicParameters })));
                    ctx.shouldOutputJson = !!flags.json;
                    ctx.shouldOutputMarkdown = !!flags.markdown;
                    ctx.config = config;
                }
            }
        ], context => ({
            renderer: context.flags.markdown ? "silent" : "default"
        }));
        const { service } = config;
        if (!service) {
            throw new Error("Service mising from config. This should have been validated elsewhere");
        }
        const { targetUrl, diffToPrevious: { changes, validationConfig } } = checkSchemaResult;
        const failures = changes.filter(({ type }) => type === graphqlTypes_1.ChangeType.FAILURE);
        if (shouldOutputJson) {
            return this.log(JSON.stringify({ targetUrl, changes, validationConfig }, null, 2));
        }
        else if (shouldOutputMarkdown) {
            const serviceName = config.service && config.service.name;
            if (!serviceName) {
                throw new Error("The service name should have been defined in the Apollo config and validated when the config was loaded. Please file an issue if you're seeing this error.");
            }
            return this.log(formatMarkdown({
                checkSchemaResult,
                serviceName,
                tag: config.tag
            }));
        }
        if (changes.length === 0) {
            return this.log("\nNo changes present between schemas\n");
        }
        this.log("\n");
        heroku_cli_util_1.table(changes.map(formatChange), {
            columns: [
                { key: "type", label: "Change" },
                { key: "code", label: "Code" },
                { key: "description", label: "Description" }
            ]
        });
        this.log("\n");
        if (targetUrl) {
            this.log(`View full details at: ${targetUrl}`);
        }
        if (failures.length > 0) {
            this.exit();
        }
        return;
    }
}
ServiceCheck.aliases = ["schema:check"];
ServiceCheck.description = "Check a service against known operation workloads to find breaking changes";
ServiceCheck.flags = Object.assign({}, Command_1.ProjectCommand.flags, { tag: command_1.flags.string({
        char: "t",
        description: "The published tag to check this service against"
    }), validationPeriod: command_1.flags.string({
        description: "The size of the time window with which to validate the schema against. You may provide a number (in seconds), or an ISO8601 format duration for more granularity (see: https://en.wikipedia.org/wiki/ISO_8601#Durations)"
    }), queryCountThreshold: command_1.flags.integer({
        description: "Minimum number of requests within the requested time window for a query to be considered."
    }), queryCountThresholdPercentage: command_1.flags.integer({
        description: "Number of requests within the requested time window for a query to be considered, relative to total request count. Expected values are between 0 and 0.05 (minimum 5% of total request volume)"
    }), json: command_1.flags.boolean({
        description: "Output result in json, which can then be parsed by CLI tools such as jq.",
        exclusive: ["markdown"]
    }), markdown: command_1.flags.boolean({
        description: "Output result in markdown.",
        exclusive: ["json"]
    }) });
exports.default = ServiceCheck;
//# sourceMappingURL=check.js.map